/*
 * DO NOT EDIT.  THIS FILE IS GENERATED FROM /builds/slave/cen-lnx-xr/build/dom/indexedDB/nsIIDBTransactionEvent.idl
 */

#ifndef __gen_nsIIDBTransactionEvent_h__
#define __gen_nsIIDBTransactionEvent_h__


#ifndef __gen_nsIIDBSuccessEvent_h__
#include "nsIIDBSuccessEvent.h"
#endif

/* For IDL files that don't want to include root IDL files. */
#ifndef NS_NO_VTABLE
#define NS_NO_VTABLE
#endif
class nsIIDBTransaction; /* forward declaration */


/* starting interface:    nsIIDBTransactionEvent */
#define NS_IIDBTRANSACTIONEVENT_IID_STR "cdf58757-78b9-4f5f-8559-1bf96e1cdfba"

#define NS_IIDBTRANSACTIONEVENT_IID \
  {0xcdf58757, 0x78b9, 0x4f5f, \
    { 0x85, 0x59, 0x1b, 0xf9, 0x6e, 0x1c, 0xdf, 0xba }}

class NS_NO_VTABLE NS_SCRIPTABLE nsIIDBTransactionEvent : public nsIIDBSuccessEvent {
 public: 

  NS_DECLARE_STATIC_IID_ACCESSOR(NS_IIDBTRANSACTIONEVENT_IID)

  /* readonly attribute nsIIDBTransaction transaction; */
  NS_SCRIPTABLE NS_IMETHOD GetTransaction(nsIIDBTransaction **aTransaction) = 0;

};

  NS_DEFINE_STATIC_IID_ACCESSOR(nsIIDBTransactionEvent, NS_IIDBTRANSACTIONEVENT_IID)

/* Use this macro when declaring classes that implement this interface. */
#define NS_DECL_NSIIDBTRANSACTIONEVENT \
  NS_SCRIPTABLE NS_IMETHOD GetTransaction(nsIIDBTransaction **aTransaction); 

/* Use this macro to declare functions that forward the behavior of this interface to another object. */
#define NS_FORWARD_NSIIDBTRANSACTIONEVENT(_to) \
  NS_SCRIPTABLE NS_IMETHOD GetTransaction(nsIIDBTransaction **aTransaction) { return _to GetTransaction(aTransaction); } 

/* Use this macro to declare functions that forward the behavior of this interface to another object in a safe way. */
#define NS_FORWARD_SAFE_NSIIDBTRANSACTIONEVENT(_to) \
  NS_SCRIPTABLE NS_IMETHOD GetTransaction(nsIIDBTransaction **aTransaction) { return !_to ? NS_ERROR_NULL_POINTER : _to->GetTransaction(aTransaction); } 

#if 0
/* Use the code below as a template for the implementation class for this interface. */

/* Header file */
class nsIDBTransactionEvent : public nsIIDBTransactionEvent
{
public:
  NS_DECL_ISUPPORTS
  NS_DECL_NSIIDBTRANSACTIONEVENT

  nsIDBTransactionEvent();

private:
  ~nsIDBTransactionEvent();

protected:
  /* additional members */
};

/* Implementation file */
NS_IMPL_ISUPPORTS1(nsIDBTransactionEvent, nsIIDBTransactionEvent)

nsIDBTransactionEvent::nsIDBTransactionEvent()
{
  /* member initializers and constructor code */
}

nsIDBTransactionEvent::~nsIDBTransactionEvent()
{
  /* destructor code */
}

/* readonly attribute nsIIDBTransaction transaction; */
NS_IMETHODIMP nsIDBTransactionEvent::GetTransaction(nsIIDBTransaction **aTransaction)
{
    return NS_ERROR_NOT_IMPLEMENTED;
}

/* End of implementation class template. */
#endif


#endif /* __gen_nsIIDBTransactionEvent_h__ */
